import pandas as pd
import numpy as np

# Set random seed for reproducibility
np.random.seed(42)

# Define the number of transactions
num_records = 2000  # Increased dataset size for better realism

# Generate customer profiles
num_customers = 500
customer_ids = np.random.choice(range(1000, 9999), num_customers, replace=False)

# Generate realistic product categories with associated price ranges
product_categories = {
    "Electronics": (100, 2000),
    "Clothing": (20, 200),
    "Grocery": (5, 50),
    "Books": (10, 100),
    "Home Appliances": (50, 800),
    "Toys": (15, 150),
    "Beauty Products": (10, 300)
}
category_names = list(product_categories.keys())

# Generate seasonal sales trends
start_date = pd.to_datetime("2023-01-01")
transaction_dates = [start_date + pd.DateOffset(days=int(np.random.normal(180, 90))) for _ in range(num_records)]
transaction_dates = np.array([date if date >= start_date else start_date for date in transaction_dates])

# Generate customer spending behavior (some spend more frequently)
customer_spending_profiles = {
    "High Spender": (300, 2000, 10),
    "Medium Spender": (50, 500, 30),
    "Low Spender": (10, 100, 60)
}
customer_segments = np.random.choice(list(customer_spending_profiles.keys()), num_customers, p=[0.2, 0.5, 0.3])
customer_profiles = dict(zip(customer_ids, customer_segments))

# Generate transaction details
transactions = []
for _ in range(num_records):
    customer_id = np.random.choice(customer_ids)
    segment = customer_profiles[customer_id]
    
    # Choose product category
    product_category = np.random.choice(category_names)
    price_range = product_categories[product_category]
    
    # Generate transaction amount based on segment behavior
    min_price, max_price, segment_prob = customer_spending_profiles[segment]
    transaction_amount = round(np.random.uniform(min_price, max_price) * np.random.uniform(0.8, 1.2), 2)
    
    # Generate discount (more discount on expensive items)
    discount = np.random.uniform(5, 20) if transaction_amount > 300 else np.random.uniform(0, 10)
    
    # Generate quantity (higher for groceries)
    quantity = np.random.randint(1, 3) if transaction_amount > 300 else np.random.randint(1, 10)
    
    # Generate payment methods (more frequent usage of digital payments)
    payment_methods = ["Credit Card", "Debit Card", "PayPal", "Cash", "Cryptocurrency"]
    payment_weights = [0.4, 0.3, 0.2, 0.08, 0.02]  # Credit cards are used the most
    payment_method = np.random.choice(payment_methods, p=payment_weights)
    
    # Append to transactions
    transactions.append([
        customer_id, transaction_amount, transaction_dates[_], product_category, payment_method, round(discount, 2), quantity
    ])

# Create DataFrame
df = pd.DataFrame(transactions, columns=[
    "Customer ID", "Transaction Amount ($)", "Transaction Date", "Product Category", "Payment Method", "Discount (%)", "Quantity"
])

# Save to CSV
df.to_csv("realistic_sales_dataset.csv", index=False)

# Display the first few rows
print(df.head())

"""
### What Makes This Dataset More Realistic?
- Customer Segments: Some customers spend more than others, leading to more realistic spending patterns.
- Seasonality in Transactions: Purchase dates follow a normal distribution around the middle of the year, resembling real-world trends.
- Product Categories & Price Ranges: Different categories have realistic price points (e.g., Electronics cost more than Books).
- Discounts Based on Price: Higher-priced products get higher discounts, similar to real retail promotions.
- Realistic Payment Method Usage: Most people use credit/debit cards, while cash and cryptocurrency are rare.
- Quantity Variation: Groceries are bought in higher quantities, while electronics tend to be single purchases.

This dataset is great for sales analysis, customer behavior modeling, and machine learning projects!
"""
